@@ -1,7 +1,26 @@
-## function for conditional likelihoods at nodes
-## written by Liam J. Revell 2015, 2016, 2019, 2020
+## written by Liam J. Revell 2015, 2016, 2019, 2020, 2021, 2022
 ## with input from (& structural similarity to) function ace by E. Paradis et al. 2013
 
+anova.fitMk<-function(object,...){
+	fits<-list(...)
+	nm<-c(
+		deparse(substitute(object)),
+		if(length(fits)>0) sapply(substitute(list(...))[-1],deparse)
+	)
+	logL<-c(logLik(object),
+		if(length(fits)>0) sapply(fits,logLik))
+	df<-c(attr(logLik(object),"df"),
+		if(length(fits)>0) sapply(fits,function(x) attr(logLik(x),"df")))
+	AICvals<-c(AIC(object),
+		if(length(fits)>0) sapply(fits,AIC))
+	ww<-aic.w(AICvals)
+	result<-data.frame(logL,df,AICvals,unclass(ww))
+	rownames(result)<-nm
+	colnames(result)<-c("log(L)","d.f.","AIC","weight")
+	print(result)
+	invisible(result)
+}
+
 fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	if(hasArg(output.liks)) output.liks<-list(...)$output.liks
 	else output.liks<-FALSE
@@ -11,6 +30,10 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	else opt.method<-"nlminb"
 	if(hasArg(min.q)) min.q<-list(...)$min.q
 	else min.q<-1e-12
+	if(hasArg(max.q)) max.q<-list(...)$max.q
+	else max.q<-max(nodeHeights(tree))*100
+	if(hasArg(logscale)) logscale<-list(...)$logscale
+	else logscale<-FALSE
 	N<-Ntip(tree)
 	M<-tree$Nnode
 	if(is.matrix(x)){
@@ -25,14 +48,24 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	}
 	if(hasArg(pi)) pi<-list(...)$pi
 	else pi<-"equal"
-	if(pi[1]=="equal") pi<-setNames(rep(1/m,m),states)
-	else if(pi[1]=="estimated"){ 
-		pi<-if(!is.null(fixedQ)) statdist(fixedQ) else statdist(summary(fitMk(tree,x,model),quiet=TRUE)$Q)
-		cat("Using pi estimated from the stationary distribution of Q assuming a flat prior.\npi =\n")
+	if(is.numeric(pi)) root.prior<-"given"
+	if(pi[1]=="equal"){ 
+		pi<-setNames(rep(1/m,m),states)
+		root.prior<-"flat"
+	} else if(pi[1]=="estimated"){ 
+		pi<-if(!is.null(fixedQ)) statdist(fixedQ) else 
+			statdist(summary(fitMk(tree,x,model),quiet=TRUE)$Q)
+		cat(paste("Using pi estimated from the stationary",
+			"distribution of Q assuming a flat prior.\npi =\n"))
 		print(round(pi,6))
 		cat("\n")
-	}	
-	else pi<-pi/sum(pi)
+		root.prior<-"stationary"
+	} else if(pi[1]=="fitzjohn") root.prior<-"nuisance"
+	if(is.numeric(pi)){ 
+		pi<-pi/sum(pi)
+		if(is.null(names(pi))) pi<-setNames(pi,states)
+		pi<-pi[states]
+	} 
 	if(is.null(fixedQ)){
 		if(is.character(model)){
 			rate<-matrix(NA,m,m)
@@ -70,7 +103,10 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	rate[rate==0]<-k+1
 	liks<-rbind(x,matrix(0,M,m,dimnames=list(1:M+N,states)))
 	pw<-reorder(tree,"pruningwise")
-	lik<-function(Q,output.liks=FALSE,pi){
+	lik<-function(Q,output.liks=FALSE,pi,...){
+		if(hasArg(output.pi)) output.pi<-list(...)$output.pi
+		else output.pi<-FALSE
+		if(is.Qmatrix(Q)) Q<-unclass(Q)
 		if(any(is.nan(Q))||any(is.infinite(Q))) return(1e50)
 		comp<-vector(length=N+M,mode="numeric")
 		parents<-unique(pw$edge[,1])
@@ -84,45 +120,73 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 			for(j in 1:length(v)){
 				v[[j]]<-EXPM(Q*el[j])%*%liks[desc[j],]
 			}
-			vv<-if(anc==root) Reduce('*',v)[,1]*pi else Reduce('*',v)[,1]
+			if(anc==root){
+				if(is.numeric(pi)) vv<-Reduce('*',v)[,1]*pi
+				else if(pi[1]=="fitzjohn"){
+					D<-Reduce('*',v)[,1]
+					pi<-D/sum(D)
+					vv<-D*D/sum(D)
+				}
+			} else vv<-Reduce('*',v)[,1]
+			## vv<-if(anc==root) Reduce('*',v)[,1]*pi else Reduce('*',v)[,1]
 			comp[anc]<-sum(vv)
 			liks[anc,]<-vv/comp[anc]
 		}
-		if(output.liks)return(liks[1:M+N,,drop=FALSE]) 
-		logL<--sum(log(comp[1:M+N]))
-		return(if(is.na(logL)) Inf else logL)
+		if(output.liks) return(liks[1:M+N,,drop=FALSE])
+		else if(output.pi) return(pi)
+		else {
+			logL<--sum(log(comp[1:M+N]))
+			if(is.na(logL)) logL<-Inf
+			return(logL)
+		}
 	}
 	if(is.null(fixedQ)){
 		if(length(q.init)!=k) q.init<-rep(q.init[1],k)
-		if(opt.method=="optim")
-			fit<-optim(q.init,function(p) lik(makeQ(m,p,index.matrix),pi=pi),
-				method="L-BFGS-B",lower=rep(min.q,k))
-		else if(opt.method=="none")
+		q.init<-if(logscale) log(q.init) else q.init
+		if(opt.method=="optim"){
+			fit<-if(logscale) 
+				optim(q.init,function(p) lik(makeQ(m,exp(p),index.matrix),pi=pi),
+					method="L-BFGS-B",lower=rep(log(min.q),k),upper=rep(log(max.q),k)) else
+				optim(q.init,function(p) lik(makeQ(m,p,index.matrix),pi=pi),
+					method="L-BFGS-B",lower=rep(min.q,k),upper=rep(max.q,k))
+		} else if(opt.method=="none"){
 			fit<-list(objective=lik(makeQ(m,q.init,index.matrix),pi=pi),
 				par=q.init)
-		else	
-			fit<-nlminb(q.init,function(p) lik(makeQ(m,p,index.matrix),pi=pi),
-				lower=rep(0,k),upper=rep(1e50,k))
+		} else {
+			fit<-if(logscale)
+				nlminb(q.init,function(p) lik(makeQ(m,exp(p),index.matrix),pi=pi),
+					lower=rep(log(min.q),k),upper=rep(log(max.q),k))
+				else nlminb(q.init,function(p) lik(makeQ(m,p,index.matrix),
+					pi=pi),lower=rep(0,k),upper=rep(max.q,k))
+		}
+		if(logscale) fit$par<-exp(fit$par)
+		if(pi[1]=="fitzjohn") pi<-setNames(
+			lik(makeQ(m,fit$par,index.matrix),FALSE,pi=pi,output.pi=TRUE),
+			states)
 		obj<-list(logLik=
 			if(opt.method=="optim") -fit$value else -fit$objective,
 			rates=fit$par,
 			index.matrix=index.matrix,
 			states=states,
 			pi=pi,
-			method=opt.method)
+			method=opt.method,
+			root.prior=root.prior)
 		if(output.liks) obj$lik.anc<-lik(makeQ(m,obj$rates,index.matrix),TRUE,
 			pi=pi)
 	} else {
 		fit<-lik(Q,pi=pi)
+		if(pi[1]=="fitzjohn") pi<-setNames(lik(Q,FALSE,pi=pi,output.pi=TRUE),states)
 		obj<-list(logLik=-fit,
 			rates=Q[sapply(1:k,function(x,y) which(x==y),index.matrix)],
 			index.matrix=index.matrix,
 			states=states,
-			pi=pi)
+			pi=pi,
+			root.prior=root.prior)
 		if(output.liks) obj$lik.anc<-lik(makeQ(m,obj$rates,index.matrix),TRUE,
 			pi=pi)
 	}
-	lik.f<-function(q) -lik(q,output.liks=FALSE,pi=pi)
+	lik.f<-function(q) -lik(q,output.liks=FALSE,
+		pi=if(root.prior=="nuisance") "fitzjohn" else pi)
 	obj$lik<-lik.f
 	class(obj)<-"fitMk"
 	return(obj)
@@ -147,9 +211,12 @@ print.fitMk<-function(x,digits=6,...){
 	colnames(Q)<-rownames(Q)<-x$states
 	print(round(Q,digits))
 	cat("\nFitted (or set) value of pi:\n")
-	print(x$pi)
+	print(round(x$pi,digits))
+	cat(paste("due to treating the root prior as (a) ",x$root.prior,".\n",
+		sep=""))
 	cat(paste("\nLog-likelihood:",round(x$logLik,digits),"\n"))
-	cat(paste("\nOptimization method used was \"",x$method,"\"\n\n",sep=""))
+	cat(paste("\nOptimization method used was \"",x$method,"\"\n\n",
+		sep=""))
 }
 
 ## summary method for objects of class "fitMk"
@@ -186,9 +253,10 @@ plot.fitMk<-function(x,...){
 plot.gfit<-function(x,...){
 	if("mkn"%in%class(x$lik)==FALSE){
 		stop("Sorry. No plot method presently available for objects of this type.")
+		object<-NULL
 	} else {
 		chk<-.check.pkg("geiger")
-		if(chk) plot(as.Qmatrix(x),...)
+		if(chk) object<-plot(as.Qmatrix(x),...)
 		else {
 			obj<-list()
 			QQ<-.Qmatrix.from.gfit(x)
@@ -199,10 +267,15 @@ plot.gfit<-function(x,...){
 			obj$index.matrix[col(obj$index.matrix)!=row(obj$index.matrix)]<-1:k
 			obj$rates<-QQ[sapply(1:k,function(x,y) which(x==y),obj$index.matrix)]
 			class(obj)<-"fitMk"
-			plot(obj,...)
+			object<-plot(obj,...)
 		}
 	}
+	invisible(object)
 }
+
+MIN<-function(x,...) min(x[is.finite(x)],...)
+MAX<-function(x,...) max(x[is.finite(x)],...)
+RANGE<-function(x,...) range(x[is.finite(x)],...)
 	
 ## S3 method for "Qmatrix" object class
 plot.Qmatrix<-function(x,...){
@@ -225,18 +298,75 @@ plot.Qmatrix<-function(x,...){
 	else mar<-c(1.1,1.1,3.1,1.1)
 	if(hasArg(lwd)) lwd<-list(...)$lwd
 	else lwd<-1
-	spacer<-0.1
+	if(hasArg(umbral)) umbral<-list(...)$umbral	
+	else umbral<-FALSE
+	if(hasArg(ncat)) ncat<-list(...)$ncat
+	else ncat<-NULL
+	if(hasArg(spacer)) spacer<-list(...)$spacer
+	else spacer<-0.1
+	if(hasArg(color)) color<-list(...)$color
+	else color<-FALSE
+	if(hasArg(width)) width<-list(...)$width
+	else width<-FALSE
+	if(hasArg(text)) text<-list(...)$text
+	else text<-TRUE
+	if(hasArg(max.lwd)) max.lwd<-list(...)$max.lwd
+	else max.lwd<-if(text) 5 else 8
 	plot.new()
 	par(mar=mar)
 	xylim<-c(-1.2,1.2)
-	plot.window(xlim=xylim,ylim=xylim,asp=1)
+	if(!color) plot.window(xlim=xylim,ylim=xylim,asp=1) else 
+		plot.window(xlim=c(-1.4,xylim[2]-0.2),ylim=xylim,asp=1)
 	if(!is.null(main)) title(main=main,cex.main=cex.main)
 	nstates<-nrow(Q)
-	step<-360/nstates
-	angles<-seq(0,360-step,by=step)/180*pi
-	if(nstates==2) angles<-angles+pi/2
-	v.x<-cos(angles)
-	v.y<-sin(angles)
+	if(color){
+		col_pal<-function(qq) if(is.na(qq)) NA else 
+			if(is.infinite(qq)) make.transparent("grey",0.6) else
+			rgb(colorRamp(c("blue","purple","red"))(qq),maxColorValue=255)
+		qq<-Q
+		diag(qq)<-NA
+		qq<-log(qq)
+		dq<-diff(RANGE(qq,na.rm=TRUE))
+		if(dq<tol){
+			cols<-matrix("blue",nstates,nstates)
+			cols[Q<tol]<-make.transparent("grey",0.6)
+		} else {
+			qq<-(qq-MIN(qq,na.rm=TRUE))/dq
+			cols<-apply(qq,c(1,2),col_pal)
+		}
+	} else cols<-matrix(par("fg"),nstates,nstates)
+	if(width){
+		lwd_maker<-function(qq,max.qq) if(is.na(qq)) NA else 
+			if(is.infinite(qq)) 0 else qq*(max.lwd-1)+1
+		qq<-Q
+		diag(qq)<-NA
+		qq<-log(qq)
+		dq<-max(qq[!is.infinite(qq)],na.rm=TRUE)-
+			min(qq[!is.infinite(qq)],na.rm=TRUE)
+		if(dq<tol){
+			lwd<-matrix(lwd,nstates,nstates)
+			lwd[Q<tol]<-0
+		} else {
+			qq<-(qq-(min(qq[!is.infinite(qq)],na.rm=TRUE)))/dq
+			lwd<-apply(qq,c(1,2),lwd_maker,max.qq=max(qq,na.rm=TRUE))
+		}
+	} else lwd<-matrix(lwd,nstates,nstates)
+	if(!umbral||is.null(ncat)){
+		step<-360/nstates
+		angles<-seq(0,360-step,by=step)/180*pi
+		if(nstates==2) angles<-angles+pi/2
+		v.x<-cos(angles)
+		v.y<-sin(angles)
+	} else {
+		v.x<-v.y<-vector()
+		for(i in 1:length(ncat)){
+			Q<-Q[sort(rownames(Q)),sort(colnames(Q))]
+			xp<--1+2*(i-1)/(length(ncat)-1)
+			v.x<-c(v.x,rep(xp,ncat[i]))
+			yp<-seq(1,-1,length.out=max(ncat))[1:ncat[i]]
+			v.y<-c(v.y,yp)
+		}
+	}	
 	for(i in 1:nstates) for(j in 1:nstates)
 		if(if(!isSymmetric(Q)) i!=j else i>j){
 			dx<-v.x[j]-v.x[i]
@@ -253,24 +383,75 @@ plot.Qmatrix<-function(x,...){
 				v.y[j]+spacer*sin(atan(slope))*sign(-dy)+
 				if(isSymmetric(Q)) 0 else shift.y)
 			if(show.zeros||Q[i,j]>tol){
-				if(abs(diff(c(i,j)))==1||abs(diff(c(i,j)))==(nstates-1))
-					text(mean(c(s[1],e[1]))+1.5*shift.x,
-						mean(c(s[2],e[2]))+1.5*shift.y,
-						round(Q[i,j],signif),cex=cex.rates,
-						srt=atan(dy/dx)*180/pi)
-				else
-					text(mean(c(s[1],e[1]))+0.3*diff(c(s[1],e[1]))+
-						1.5*shift.x,
-						mean(c(s[2],e[2]))+0.3*diff(c(s[2],e[2]))+
-						1.5*shift.y,
-						round(Q[i,j],signif),cex=cex.rates,
-						srt=atan(dy/dx)*180/pi)
+				if(text){
+					if(abs(diff(c(i,j)))==1||abs(diff(c(i,j)))==(nstates-1))
+						text(mean(c(s[1],e[1]))+1.5*shift.x,
+							mean(c(s[2],e[2]))+1.5*shift.y,
+							round(Q[i,j],signif),cex=cex.rates,
+							srt=atan(dy/dx)*180/pi)
+					else
+						text(mean(c(s[1],e[1]))+0.3*diff(c(s[1],e[1]))+
+							1.5*shift.x,
+							mean(c(s[2],e[2]))+0.3*diff(c(s[2],e[2]))+
+							1.5*shift.y,
+							round(Q[i,j],signif),cex=cex.rates,
+							srt=atan(dy/dx)*180/pi)
+				}
 				arrows(s[1],s[2],e[1],e[2],length=0.05,
-					code=if(isSymmetric(Q)) 3 else 2,lwd=lwd)
+					code=if(isSymmetric(Q)) 3 else 2,
+					lwd=if(lwd[i,j]==0) 1 else lwd[i,j],
+					lty=if(lwd[i,j]==0) "dotted" else "solid",
+					col=cols[i,j])
 			}
 		}
 	text(v.x,v.y,rownames(Q),cex=cex.traits,
-		col=make.transparent("black",0.7))
+		col=make.transparent(par("fg"),0.9))
+	if(color){
+		if(dq>tol){
+			h<-1.5
+			LWD<-diff(par()$usr[1:2])/dev.size("px")[1]
+			lines(x=rep(-1.3+LWD*15/2,2),y=c(-h/2,h/2))
+			nticks<-6
+			Y<-cbind(seq(-h/2,h/2,length.out=nticks),
+				seq(-h/2,h/2,length.out=nticks))
+			X<-cbind(rep(-1.3+LWD*15/2,nticks),
+				rep(-1.3+LWD*15/2+0.02*h,nticks))
+			for(i in 1:nrow(Y)) lines(X[i,],Y[i,])
+			add.color.bar(h,sapply(seq(0,1,length.out=100),col_pal),
+				title="evolutionary rate (q)",
+				lims=NULL,digits=3,
+				direction="upwards",
+				subtitle="",lwd=15,
+				x=-1.3,y=-h/2,prompt=FALSE)
+			QQ<-Q
+			diag(QQ)<-0
+			text(x=X[,2],y=Y[,2],signif(exp(seq(MIN(log(QQ),na.rm=TRUE),
+				MAX(log(QQ),na.rm=TRUE),length.out=6)),signif),pos=4,cex=0.7)
+		} else {
+			BLUE<-function(...) "blue"
+			h<-1.5
+			LWD<-diff(par()$usr[1:2])/dev.size("px")[1]
+			lines(x=rep(-1.3+LWD*15/2,2),y=c(-h/2,h/2))
+			nticks<-6
+			Y<-cbind(seq(-h/2,h/2,length.out=nticks),
+				seq(-h/2,h/2,length.out=nticks))[nticks,,drop=FALSE]
+			X<-cbind(rep(-1.3+LWD*15/2,nticks),
+				rep(-1.3+LWD*15/2+0.02*h,nticks))[nticks,,drop=FALSE]
+			for(i in 1:nrow(Y)) lines(X[i,],Y[i,])
+			add.color.bar(h,sapply(seq(0,1,length.out=100),BLUE),
+				title="evolutionary rate (q)",
+				lims=NULL,digits=3,
+				direction="upwards",
+				subtitle="",lwd=15,
+				x=-1.3,y=-h/2,prompt=FALSE)
+			QQ<-Q
+			diag(QQ)<-0
+			text(x=X[,2],y=Y[,2],signif(exp(seq(MIN(log(QQ),na.rm=TRUE),
+				MAX(log(QQ),na.rm=TRUE),length.out=1)),signif),pos=4,cex=0.7)
+		}
+	}
+	object<-data.frame(states=rownames(Q),x=v.x,y=v.y)
+	invisible(object)
 }
 
 ## wraps around expm
@@ -360,13 +541,27 @@ as.Qmatrix.fitMk<-function(x,...){
 	Q<-matrix(NA,length(x$states),length(x$states))
 	Q[]<-c(0,x$rates)[x$index.matrix+1]
 	rownames(Q)<-colnames(Q)<-x$states
-	diag(Q)<-0
-	diag(Q)<--rowSums(Q)
+	diag(Q)<--rowSums(Q,na.rm=TRUE)
 	class(Q)<-"Qmatrix"
 	Q
 }
 
+as.Qmatrix.ace<-function(x, ...){
+	if("index.matrix"%in%names(x)){
+		k<-nrow(x$index.matrix)
+		Q<-matrix(NA,k,k)
+		Q[]<-c(0,x$rates)[x$index.matrix+1]
+		rownames(Q)<-colnames(Q)<-colnames(x$lik.anc)
+		diag(Q)<--rowSums(Q,na.rm=TRUE)
+		class(Q)<-"Qmatrix"
+		return(Q)
+	} else cat("\"ace\" object does not appear to contain a Q matrix.\n")
+}
+
 print.Qmatrix<-function(x,...){
 	cat("Estimated Q matrix:\n")
-	print(unclass(x))
+	print(unclass(x),...)
 }
+
+is.Qmatrix<-function(x) "Qmatrix" %in% class(x)
+
