@@ -1,5 +1,28 @@
 ## some utility functions
-## written by Liam J. Revell 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021
+## written by Liam J. Revell 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022
+
+## function to get states at internal nodes from simmap style trees
+## written by Liam J. Revell 2013, 2014, 2015, 2021
+getStates<-function(tree,type=c("nodes","tips","both")){
+	type<-type[1]
+	if(inherits(tree,"multiPhylo")){
+		tree<-unclass(tree)
+		obj<-lapply(tree,getStates,type=type)
+		nn<-names(obj[[1]])
+		y<-sapply(obj,function(x,n) x[n],n=nn)
+	} else if(inherits(tree,"phylo")){ 
+		if(type%in%c("nodes","both")){
+			a<-setNames(sapply(tree$maps,function(x) names(x)[1]),tree$edge[,1])
+			a<-a[as.character(Ntip(tree)+1:tree$Nnode)]
+		}
+		if(type%in%c("tips","both")){
+			b<-setNames(sapply(tree$maps,function(x) names(x)[length(x)]),tree$edge[,2])
+			b<-setNames(b[as.character(1:Ntip(tree))],tree$tip.label)
+		}
+		y<-if(type=="both") c(a,b) else if(type=="nodes") a else b
+	} else stop("tree should be an object of class \"phylo\" or \"multiPhylo\".")
+	return(y)
+}
 
 ## di2multi & multi2di for "contMap" & "densityMap" object classes
 
@@ -384,10 +407,10 @@ force.ultrametric<-function(tree,method=c("nnls","extend"),...){
 }
 
 ## function to create curved clade labels for a fan tree
-## written by Liam J. Revell 2017
+## written by Liam J. Revell 2017, 2022
 
 arc.cladelabels<-function(tree=NULL,text,node=NULL,ln.offset=1.02,
-	lab.offset=1.06,cex=1,orientation="curved",...){
+	lab.offset=1.06,cex=1,orientation="curved",stretch=1,...){
 	obj<-get("last_plot.phylo",envir=.PlotPhyloEnv)
 	if(obj$type!="fan") stop("method works only for type=\"fan\"")
 	h<-max(sqrt(obj$xx^2+obj$yy^2))
@@ -430,7 +453,7 @@ arc.cladelabels<-function(tree=NULL,text,node=NULL,ln.offset=1.02,
 	if(orientation=="curved")
 		arctext(text,radius=lab.offset*h,
 			middle=mean(range(deg*pi/180)),cex=cex,
-			clockwise=clockwise)
+			clockwise=clockwise,stretch=stretch)
 	else if(orientation=="horizontal"){
 		x0<-lab.offset*cos(median(deg)*pi/180)*h
 		y0<-lab.offset*sin(median(deg)*pi/180)*h
@@ -504,7 +527,7 @@ labelnodes<-function(text,node=NULL,interactive=TRUE,
 
 ## convert object of class "birthdeath" into birth & death rates
 bd<-function(x){
-	if(class(x)!="birthdeath") stop("x should be an object of class 'birthdeath'")
+	if(!inherits(x,"birthdeath")) stop("x should be an object of class 'birthdeath'")
 	b<-x$para[2]/(1-x$para[1])
 	d<-b-x$para[2]
 	setNames(c(b,d),c("b","d"))
@@ -1270,8 +1293,8 @@ plot.describe.simmap<-function(x,...){
 }
 
 
-# function to summarize the results of stochastic mapping
-# written by Liam J. Revell 2013, 2014, 2015
+## function to summarize the results of stochastic mapping
+## written by Liam J. Revell 2013, 2014, 2015, 2021
 describe.simmap<-function(tree,...){
 	if(hasArg(plot)) plot<-list(...)$plot
 	else plot<-FALSE
@@ -1287,7 +1310,7 @@ describe.simmap<-function(tree,...){
 			check<-all(TT)
 			if(!check) cat("Note: Some trees are not equal.\nA \"reference\" tree will be computed if none was provided.\n\n")
 		} else check<-TRUE
-		YY<-getStates(tree)
+		YY<-getStates(tree,"both")
 		states<-sort(unique(as.vector(YY)))
 		if(is.null(ref.tree)&&check) ZZ<-t(apply(YY,1,function(x,levels,Nsim) summary(factor(x,levels))/Nsim,levels=states,Nsim=length(tree)))
 		else {
@@ -1806,27 +1829,6 @@ getCladesofSize<-function(tree,clade.size=2){
 	return(trees)
 }
 
-# function to get states at internal nodes from simmap style trees
-# written by Liam J. Revell 2013, 2014, 2015
-getStates<-function(tree,type=c("nodes","tips")){
-	type<-type[1]
-	if(inherits(tree,"multiPhylo")){
-		tree<-unclass(tree)
-		obj<-lapply(tree,getStates,type=type)
-		nn<-names(obj[[1]])
-		y<-sapply(obj,function(x,n) x[n],n=nn)
-	} else if(inherits(tree,"phylo")){ 
-		if(type=="nodes"){
-			y<-setNames(sapply(tree$maps,function(x) names(x)[1]),tree$edge[,1])
-			y<-y[as.character(Ntip(tree)+1:tree$Nnode)]
-		} else if(type=="tips"){
-			y<-setNames(sapply(tree$maps,function(x) names(x)[length(x)]),tree$edge[,2])
-			y<-setNames(y[as.character(1:Ntip(tree))],tree$tip.label)
-		}
-	} else stop("tree should be an object of class \"phylo\" or \"multiPhylo\".")
-	return(y)
-}
-
 # function counts transitions from a mapped history
 # written by Liam J. Revell 2013, 2015
 countSimmap<-function(tree,states=NULL,message=TRUE){
